<?php

require_once "abstract.ilPluginControllerTableGUI.php";

/**
 * Class ilQuestionSetPoolSetsGUI
 *
 * Date: 02.10.13
 * Time: 09:33
 * @author Thomas Joußen <tjoussen@databay.de>
 */ 
class ilQuestionSetPoolSetsGUI extends ilPluginControllerTableGUI
{

	public function show()
	{
		$this->addToolbarButton("add_set", "ilQuestionSetPoolSetGUI.edit");

		$table = new ilQuestionSetPoolSetsTableGUI($this->controller, $this->plugin);
		$tmp = $this->controller->object->getSetList();
		$table->setModelList($this->controller->object->getSetList());
		$table->addMultiCommand("ilQuestionSetPoolSetsGUI.duplicate", $this->lng->txt("copy"));
		$table->populate();

		$this->tpl->setContent($table->getHTML());
	}

	/**
	 * Called if a single set should be deleted, calls \ilQuestionSetPoolSetsGUI::removeMultipleConfirmation do show the confirmation screen
	 */
	public function removeConfirmation()
	{
		if(!isset($_GET['set_id']))
		{
			ilUtil::sendInfo($this->plugin->txt('no_set_selected'));
			$this->show();
			return;
		}

		$_POST['set_id'] = array((int)$_GET['set_id']);
		$this->removeMultipleConfirmation();
	}

	/**
	 * Shows an confirmation screen for the selected sets
	 */
	public function removeMultipleConfirmation()
	{
		if(!isset($_POST['set_id']) || !is_array($_POST['set_id']))
		{
			ilUtil::sendInfo($this->plugin->txt('no_set_selected'));
			$this->show();
			return;
		}

		$this->tabs->activateSubTab('paths');

		require_once 'Services/Utilities/classes/class.ilConfirmationGUI.php';
		$confirmation = new ilConfirmationGUI();

		$num_items                 = 0;
		$missing_perm_nodes_titles = array();
		foreach($_POST['set_id'] as $set_id)
		{
			$set = new ilQuestionSetPoolSet((int)$set_id);
			$set->read();
			if($set->getContainerObjId() == $this->controller->object->getId())
			{
				$confirmation->addItem('set_id[]', $set->getId(), $set->getTitle());
				++$num_items;
			}
			else
			{
				$missing_perm_nodes_titles[] = $set->getTitle();
			}
		}

		if(1 == $num_items)
		{
			$confirmation->setHeaderText($this->plugin->txt('confirm_delete_sets_s'));
		}
		else
		{
			$confirmation->setHeaderText($this->plugin->txt('confirm_delete_sets_p'));
		}

		$missing_perm_nodes_titles = array_filter($missing_perm_nodes_titles);
		if(1 == count($missing_perm_nodes_titles))
		{
			ilUtil::sendFailure(sprintf($this->plugin->txt('sets_del_perm_err_s'), implode(', ', $missing_perm_nodes_titles)));
		}
		else if(count($missing_perm_nodes_titles) > 1)
		{
			ilUtil::sendFailure(sprintf($this->plugin->txt('sets_del_perm_err_p'), implode(', ', $missing_perm_nodes_titles)));
		}

		if(!$num_items)
		{
			$this->show();
			return;
		}

		$confirmation->setFormAction($this->ctrl->getFormAction($this->controller, __CLASS__ . '.show'));
		$confirmation->setCancel($this->plugin->txt('cancel'), __CLASS__ . '.show');
		$confirmation->setConfirm($this->plugin->txt('confirm'), __CLASS__ . '.remove');

		$this->tpl->setContent($confirmation->getHTML());
	}

	/**
	 * Finally deletes (cascading) the selected sets
	 */
	public function remove()
	{
		if(!isset($_POST['set_id']) || !is_array($_POST['set_id']))
		{
			ilUtil::sendInfo($this->plugin->txt('no_set_selected'));
			$this->show();
			return;
		}

		$num_items                 = 0;
		$missing_perm_nodes_titles = array();
		foreach($_POST['set_id'] as $set_id)
		{
			$set = new ilQuestionSetPoolSet((int)$set_id);
			$set->read();
			if($set->getContainerObjId() == $this->controller->object->getId())
			{
				$set->remove();
				++$num_items;
			}
			else
			{
				$missing_perm_nodes_titles[] = $set->getTitle();
			}
		}

		if(1 == $num_items)
		{
			ilUtil::sendSuccess($this->plugin->txt('deleted_sets_s'), true);
		}
		else if($num_items > 1)
		{
			ilUtil::sendSuccess($this->plugin->txt('deleted_sets_p'), true);
		}

		$missing_perm_nodes_titles = array_filter($missing_perm_nodes_titles);
		if(1 == count($missing_perm_nodes_titles))
		{
			ilUtil::sendFailure(sprintf($this->plugin->txt('sets_del_perm_err_s'), implode(', ', $missing_perm_nodes_titles)));
		}
		else if(count($missing_perm_nodes_titles) > 1)
		{
			ilUtil::sendFailure(sprintf($this->plugin->txt('sets_del_perm_err_p'), implode(', ', $missing_perm_nodes_titles)));
		}

		$this->redirect(__CLASS__ . '.show');
	}

	public function duplicate()
	{
		if(!$this->dataSelected("set_id"))
		{
			ilUtil::sendInfo($this->plugin->txt('no_set_selected'));
			$this->redirect("ilQuestionSetPoolSetsGUI.show");
		}

		foreach($_POST["set_id"] as $set_id)
		{
			$set = new ilQuestionSetPoolSet((int)$set_id);
			$set->read();
			$set->duplicate($this->controller->object->getId());
		}

		ilUtil::sendSuccess($this->plugin->txt("sets_duplicated"), true);
		$this->redirect("ilQuestionSetPoolSetsGUI.show");
	}


	public function synchronize()
	{
		$set = new ilQuestionSetPoolSet((int)$_GET["set_id"]);
		$set->read();

		if(ilObjQuestionSetPool::canSynchronizeSet($set->getOriginalId()))
		{
			$original = new ilQuestionSetPoolSet($set->getOriginalId());
			$original->read();
			$old_questions = $original->getQuestionList()->getData();
			$original->remove();

			$clone = $set->synchronize($original->getContainerObjId(), $old_questions);
			//$clone = $set->duplicate($original->getContainerObjId());
			$clone->setOriginalId(null);
			$clone->update();

			ilQuestionSetPoolSet::updateOriginalId($set->getOriginalId(), $clone->getId());

			ilUtil::sendSuccess($this->plugin->txt("sets_synchronize_success"), true);
		}
		$this->redirect("ilQuestionSetPoolPathsGUI.listPaths", array("set_id" => $_GET["set_id"]));
	}

	/**
	 * Returns the current tab name
	 * return string
	 */
	protected function getCurrentTab()
	{
		return "content";
	}

	/**
	 * @return array
	 */
	protected function getRequirements()
	{
		return array(
			"tables/class.ilQuestionSetPoolSetsTableGUI.php",
			"providers/class.ilQuestionSetPoolSetsProvider.php",
			"models/class.ilQuestionSetPoolSet.php"
		);
	}
}
