<?php

require_once "abstract.ilPluginModel.php";

/**
 * Class ilQuestionSetPoolSet
 *
 * Date: 02.10.13
 * Time: 12:18
 * @author Thomas Joußen <tjoussen@databay.de>
 */ 
class ilQuestionSetPoolSet extends ilPluginModel{

	/**
	 * @var integer
	 */
	protected $id;

	/**
	 * @var string
	 */
	protected $title;

	/**
	 * @var string
	 */
	protected $description;

	/**
	 * @var ilQuestionSetPoolSetQuestionList
	 */
	protected $question_list;

	/**
	 * @var ilQuestionSetPoolPathList
	 */
	protected $path_list;

	/**
	 * @var int|null
	 */
	protected $original_id;

	/**
	 * @var int
	 */
	protected $container_obj_id;
	
	/**
	 * ilQuestionSetPoolSet constructor.
	 * @param int $id question_set_id 
	 */
	public function __construct($id = 0)
	{
		require_once 'class.ilQuestionSetPoolSetQuestionList.php';
		require_once 'class.ilQuestionSetPoolPathList.php';
		$this->question_list = new ilQuestionSetPoolSetQuestionList($this);
		$this->path_list = new ilQuestionSetPoolPathList($this);

		parent::__construct($id);
	}

	/**
	 * @param integer $id
	 * @return bool
	 */
	public static function exists($id)
	{
		/**
		 * @var $ilDB ilDB
		 */
		global $ilDB;

		$result = $ilDB->queryF(
			"SELECT COUNT(*) cnt FROM rep_robj_xqsp_qs WHERE id = %s",
			array("integer"),
			array($id)
		);
		$data = $ilDB->fetchAssoc($result);

		return (int)$data['cnt'] == 1 ? true : false;
	}

	/**
	 * @param string $description
	 */
	public function setDescription($description)
	{
		$this->description = $description;
	}

	/**
	 * @return string
	 */
	public function getDescription()
	{
		return $this->description;
	}

	/**
	 * @param int $id
	 */
	public function setId($id)
	{
		$this->id = $id;
	}

	/**
	 * @return int
	 */
	public function getId()
	{
		return $this->id;
	}

	/**
	 * @param string $title
	 */
	public function setTitle($title)
	{
		$this->title = $title;
	}

	/**
	 * @return string
	 */
	public function getTitle()
	{
		return $this->title;
	}

	/**
	 * @return ilQuestionSetPoolSetQuestionList
	 */
	public function getQuestionList()
	{
		return $this->question_list;
	}

	/**
	 * @return ilQuestionSetPoolPathList
	 */
	public function getPathList()
	{
		return $this->path_list;
	}

	/**
	 * @param int|null $original_id
	 */
	public function setOriginalId($original_id)
	{
		$this->original_id = $original_id;
	}

	/**
	 * @return int|null
	 */
	public function getOriginalId()
	{
		return $this->original_id;
	}

	/**
	 * @param int $container_obj_id
	 */
	public function setContainerObjId($container_obj_id)
	{
		$this->container_obj_id = $container_obj_id;
	}

	/**
	 * @return int
	 */
	public function getContainerObjId()
	{
		return $this->container_obj_id;
	}

	public function update()
	{
		/** @var ilDB $ilDB */
		global $ilDB;

		if($this->id > 0)
		{
			return (bool)$ilDB->update(
				"rep_robj_xqsp_qs",
				array(
					"title" => array("text", $this->getTitle()),
					"description" => array("text", $this->getDescription()),
					"original_id" => array("integer", $this->getOriginalId())
				),
				array(
					"id" => array("integer", $this->getId())
				)
			);
		}
		else
		{
			$this->setId($ilDB->nextId("rep_robj_xqsp_qs"));
			return $ilDB->insert(
				"rep_robj_xqsp_qs",
				array(
					"id" => array("integer", $this->getId()),
					"title" => array("text", $this->getTitle()),
					"description" => array("text", $this->getDescription()),
					"questionsetpool_fi" => array("integer", $this->getContainerObjId()),
					"original_id" => array("integer", $this->getOriginalId())
				)
			);
		}
	}

	public function read()
	{
		global $ilDB;

		$result = $ilDB->queryF(
			"SELECT * FROM rep_robj_xqsp_qs WHERE id = %s",
			array("integer"),
			array($this->id)
		);

		$data = $ilDB->fetchAssoc($result);

		if($data != null)
		{
			$this->setContainerObjId($data["questionsetpool_fi"]);
			$this->setDescription($data['description']);
			$this->setTitle($data['title']);
			$this->setOriginalId($data["original_id"]);
		}
	}

	/**
	 * @return bool
	 */
	public function remove()
	{
		global $ilDB;

		$this->removePaths();
		$this->removeQuestions();

		return $ilDB->manipulateF(
			"DELETE FROM rep_robj_xqsp_qs WHERE id = %s",
			array("integer"),
			array($this->getId())
		);
	}

	public function isTitleUnique()
	{
		/** @var ilDB $ilDB */
		global $ilDB;
		$result = $ilDB->queryF(
			"SELECT COUNT(id) num FROM rep_robj_xqsp_qs WHERE title = %s AND id != %s AND questionsetpool_fi = %s",
			array("text", "integer", "integer"),
			array($this->getTitle(), $this->getId(), $this->getContainerObjId())
		);

		$row = $ilDB->fetchAssoc($result);

		return $row["num"] == 0;
	}

	/**
	 * @return array
	 */
	public function getPathTitles()
	{
		/** @var ilDB $ilDB */
		global $ilDB;
		
		$titles = array();
		$result = $ilDB->queryF(
			"SELECT DISTINCT(title) FROM rep_robj_xqsp_path WHERE questionset_fi = %s",
			array("integer"),
			array($this->getId())
		);

		while($row = $ilDB->fetchAssoc($result))
		{
			$titles[] = $row['title'];
		}
		
		return $titles;
	}

	/**
	 * @param string$original_title
	 * @param integer $container_obj_id
	 */
	protected function getUniqueTitleForContainer($original_title, $container_obj_id)
	{
		/**
		 * @var $ilDB ilDB
		 */
		global $ilDB;
		
		$title = $original_title;
		$i     = 1;
		if($this->getContainerObjId() == $container_obj_id)
		{
			++$i;
			$title .= ' (' . $i . ')';
		}
		
		do
		{
			$result = $ilDB->queryF(
				"SELECT id FROM rep_robj_xqsp_qs WHERE title = %s AND questionsetpool_fi = %s",
				array("text", "integer"),
				array($title, $container_obj_id)
			);
			if(!$ilDB->numRows($result))
			{
				return $title;
			}
			$title = $original_title . ' (' . ++$i . ')';
		}
		while(true);
	}

	/**
	 * @param int $container_obj_id
	 *
	 * @return self
	 */
	public function duplicate($container_obj_id)
	{
		$clone = $this->duplicateSet($container_obj_id);
		$question_ids = $this->duplicateQuestions($container_obj_id, $clone);
		$this->duplicatePaths($clone, $question_ids);

		return $clone;
	}

	public function synchronize($container_obj_id, $old_questions)
	{
		$clone = $this->duplicateSet($container_obj_id);

		$question_ids = $this->synchronizeQuestions($container_obj_id, $clone, $old_questions);
		$this->duplicatePaths($clone, $question_ids);

		return $clone;
	}

	/**
	 * @param ilPropertyFormGUI $form
	 */
	public function bindForm(ilPropertyFormGUI $form)
	{
		$this->setTitle($form->getInput('title'));
		$this->setDescription($form->getInput('description'));
	}

	/**
	 * @param int $old_id
	 * @param int $new_id
	 */
	public static function updateOriginalId($old_id, $new_id)
	{
		/** @var ilDB $ilDB */
		global $ilDB;

		$ilDB->update(
			"rep_robj_xqsp_qs",
			array("original_id" => array("integer", $new_id)),
			array("original_id" => array("integer", $old_id))
		);
	}

	/**
	 * @return array
	 */
	private function removeQuestions()
	{
		$data         = $this->getQuestionList()->getData();
		$question_ids = array();
		foreach ($data as $element)
		{
			$question_ids[] = $element["question_id"];
		}
		return $this->getQuestionList()->remove($question_ids);
	}

	private function removePaths()
	{
		require_once "class.ilQuestionSetPoolPath.php";

		$data = $this->getPathList()->getData();
		foreach ($data as $element)
		{
			$path = new ilQuestionSetPoolPath($element["id"]);
			$path->read();
			$path->remove();
		}
	}

	/**
	 * @param $container_obj_id
	 *
	 * @return ilQuestionSetPoolSet
	 */
	private function duplicateSet($container_obj_id)
	{
		$clone = new ilQuestionSetPoolSet();
		$clone->setTitle($this->getUniqueTitleForContainer($this->getTitle(), $container_obj_id));
		$clone->setDescription($this->getDescription());
		//$clone->setOriginalId($this->getOriginalId());
		$clone->setOriginalId($this->getId());
		$clone->setContainerObjId($container_obj_id);
		$clone->update();

		return $clone;
}

	/**
	 * @param $clone
	 * @param $question_ids
	 */
	private function duplicatePaths($clone, $question_ids)
	{
		require_once "class.ilQuestionSetPoolPath.php";

		$data = $this->getPathList()->getData();
		foreach($data as $element)
		{
			$path = new ilQuestionSetPoolPath($element["id"]);
			$path->read();
			$path->setSet($clone);
			$path->duplicate($question_ids);
		}
	}

	/**
	 * @param int $container_obj_id
	 * @param ilQuestionSetPoolSet $clone
	 *
	 * @return array
	 */
	private function duplicateQuestions($container_obj_id, $clone)
	{
		$data         = $this->getQuestionList()->getData();
		$question_ids = array();
		foreach($data as $element)
		{
			$question_ids[] = $element["question_id"];
		}
		$question_ids = $clone->getQuestionList()->duplicate($question_ids, $this, $container_obj_id);

		return $question_ids;
	}

	/**
	 * @param int $container_obj_id
	 * @param ilQuestionSetPoolSet $clone
	 * @param array $old_questions
	 *
	 * @return array
	 */
	private function synchronizeQuestions($container_obj_id, $clone, $old_questions)
	{
		$data = $this->getQuestionList()->getData();
		$question_ids = array();
		foreach($data as $element)
		{
			$question_ids[] = $element["question_id"];
		}
		$question_ids = $clone->getQuestionList()->synchronize($question_ids, $this, $container_obj_id, $old_questions);

		return $question_ids;
	}
}
